==> org.eclipse.papyrus.designer.transformation.languages.java.library.source_2.1.0.202201261931.jar: org/eclipse/papyrus/designer/transformation/languages/java/library/statemachine/ChangeEventTransformation.java <==
/**
 * Copyright (c) 2016, 2021 CEA LIST.
 * 
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *  Van Cam Pham        <VanCam.PHAM@cea.fr>
 *  Ported from C++ code - Bug 568883
 */
package org.eclipse.papyrus.designer.transformation.languages.java.library.statemachine;

import org.eclipse.papyrus.designer.transformation.languages.java.library.statemachine.CDefinitions;
import org.eclipse.papyrus.designer.transformation.languages.java.library.statemachine.SM2ClassesTransformationCore;
import org.eclipse.papyrus.designer.transformation.library.statemachine.SMCodeGeneratorConstants;
import org.eclipse.papyrus.designer.transformation.library.statemachine.TransformationUtil;
import org.eclipse.uml2.uml.ChangeEvent;
import org.eclipse.uml2.uml.OpaqueExpression;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.ValueSpecification;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.Extension;
import org.eclipse.xtext.xbase.lib.IterableExtensions;

@SuppressWarnings("all")
public class ChangeEventTransformation {
  @Extension
  protected CDefinitions cdefs;
  
  private SM2ClassesTransformationCore core;
  
  private org.eclipse.uml2.uml.Class superContext;
  
  public ChangeEventTransformation(final SM2ClassesTransformationCore core) {
    this.core = core;
    this.superContext = core.superContext;
    this.cdefs = core.cdefs;
  }
  
  public void createChangeEvents() {
    boolean _isEmpty = this.core.changeEvents.isEmpty();
    if (_isEmpty) {
      return;
    }
    String WAIT_TIME = "10";
    Operation changeEventOp = this.superContext.createOwnedOperation(SMCodeGeneratorConstants.CHANGE_EVENT_LISTEN_FUNCTION, null, null);
    changeEventOp.createOwnedParameter("id", this.core.intType);
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("struct timeval tv;");
    _builder.newLine();
    _builder.append("struct timespec ts;");
    _builder.newLine();
    _builder.append("bool oldValue;");
    _builder.newLine();
    _builder.append("bool newValue;");
    _builder.newLine();
    _builder.append("bool isFirstTime = true;");
    _builder.newLine();
    _builder.append("bool periodic = ");
    _builder.append(WAIT_TIME);
    _builder.append("; //ms");
    _builder.newLineIfNotEmpty();
    _builder.append("while(true) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("switch(id) {");
    _builder.newLine();
    {
      for(final ChangeEvent ch : this.core.changeEvents) {
        _builder.append("\t\t");
        _builder.append("case ");
        String _eventID = TransformationUtil.eventID(ch);
        _builder.append(_eventID, "\t\t");
        _builder.append(":");
        _builder.newLineIfNotEmpty();
        _builder.append("\t\t");
        _builder.append("\t");
        _builder.append("newValue = ");
        String _changeEventExpression = this.getChangeEventExpression(ch);
        _builder.append(_changeEventExpression, "\t\t\t");
        _builder.append(";");
        _builder.newLineIfNotEmpty();
        _builder.append("\t\t");
        _builder.append("\t");
        _builder.append("break;");
        _builder.newLine();
      }
    }
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("if (isFirstTime) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("oldValue = newValue;");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("isFirstTime = false;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("} else {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("#ifdef WIN32");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("Sleep(periodic);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("#elif _POSIX_C_SOURCE >= 199309L");
    _builder.newLine();
    _builder.append("\t\t    ");
    _builder.append("struct timespec ts;");
    _builder.newLine();
    _builder.append("\t\t    ");
    _builder.append("ts.tv_sec = periodic / 1000;");
    _builder.newLine();
    _builder.append("\t\t    ");
    _builder.append("ts.tv_nsec = (periodic % 1000) * 1000000;");
    _builder.newLine();
    _builder.append("\t\t    ");
    _builder.append("nanosleep(&ts, NULL);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("#else");
    _builder.newLine();
    _builder.append("\t\t    ");
    _builder.append("usleep(periodic * 1000);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("#endif");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("if (oldValue != newValue) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("//the expression is changed, push an event to the queue");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append(SMCodeGeneratorConstants.EVENT_QUEUE, "\t\t");
    _builder.append(".push(statemachine::PRIORITY_2, NULL, id, statemachine::CHANGE_EVENT, 0);");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("oldValue = newValue;");
    _builder.newLine();
    _builder.append("}");
    this.core.createOpaqueBehavior(this.superContext, changeEventOp, _builder.toString());
  }
  
  public String getChangeEventExpression(final ChangeEvent e) {
    if (((e.getChangeExpression() == null) || (!(e.getChangeExpression() instanceof OpaqueExpression)))) {
      return "false";
    } else {
      ValueSpecification _changeExpression = e.getChangeExpression();
      return IterableExtensions.<String>head(((OpaqueExpression) _changeExpression).getBodies());
    }
  }
}

==> org.eclipse.papyrus.designer.transformation.languages.java.library.source_3.1.0.202407120007.jar: org/eclipse/papyrus/designer/transformation/languages/java/library/statemachine/ChangeEventTransformation.java <==
/**
 * Copyright (c) 2016, 2021 CEA LIST.
 * 
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *  Van Cam Pham        <VanCam.PHAM@cea.fr>
 *  Ported from C++ code - Bug 568883
 */
package org.eclipse.papyrus.designer.transformation.languages.java.library.statemachine;

import org.eclipse.papyrus.designer.transformation.library.statemachine.SMCodeGeneratorConstants;
import org.eclipse.papyrus.designer.uml.tools.utils.StateMachineUtils;
import org.eclipse.uml2.uml.ChangeEvent;
import org.eclipse.uml2.uml.OpaqueExpression;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.ValueSpecification;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.Extension;
import org.eclipse.xtext.xbase.lib.IterableExtensions;

@SuppressWarnings("all")
public class ChangeEventTransformation {
  @Extension
  protected CDefinitions cdefs;

  private SM2ClassesTransformationCore core;

  private org.eclipse.uml2.uml.Class superContext;

  public ChangeEventTransformation(final SM2ClassesTransformationCore core) {
    this.core = core;
    this.superContext = core.superContext;
    this.cdefs = core.cdefs;
  }

  public void createChangeEvents() {
    boolean _isEmpty = this.core.changeEvents.isEmpty();
    if (_isEmpty) {
      return;
    }
    String WAIT_TIME = "10";
    Operation changeEventOp = this.superContext.createOwnedOperation(SMCodeGeneratorConstants.CHANGE_EVENT_LISTEN_FUNCTION, null, null);
    changeEventOp.createOwnedParameter("id", this.core.intType);
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("struct timeval tv;");
    _builder.newLine();
    _builder.append("struct timespec ts;");
    _builder.newLine();
    _builder.append("bool oldValue;");
    _builder.newLine();
    _builder.append("bool newValue;");
    _builder.newLine();
    _builder.append("bool isFirstTime = true;");
    _builder.newLine();
    _builder.append("bool periodic = ");
    _builder.append(WAIT_TIME);
    _builder.append("; //ms");
    _builder.newLineIfNotEmpty();
    _builder.append("while(true) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("switch(id) {");
    _builder.newLine();
    {
      for(final ChangeEvent ch : this.core.changeEvents) {
        _builder.append("\t\t");
        _builder.append("case ");
        String _eventID = StateMachineUtils.eventID(ch);
        _builder.append(_eventID, "\t\t");
        _builder.append(":");
        _builder.newLineIfNotEmpty();
        _builder.append("\t\t");
        _builder.append("\t");
        _builder.append("newValue = ");
        String _changeEventExpression = this.getChangeEventExpression(ch);
        _builder.append(_changeEventExpression, "\t\t\t");
        _builder.append(";");
        _builder.newLineIfNotEmpty();
        _builder.append("\t\t");
        _builder.append("\t");
        _builder.append("break;");
        _builder.newLine();
      }
    }
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("if (isFirstTime) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("oldValue = newValue;");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("isFirstTime = false;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("} else {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("#ifdef WIN32");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("Sleep(periodic);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("#elif _POSIX_C_SOURCE >= 199309L");
    _builder.newLine();
    _builder.append("\t\t    ");
    _builder.append("struct timespec ts;");
    _builder.newLine();
    _builder.append("\t\t    ");
    _builder.append("ts.tv_sec = periodic / 1000;");
    _builder.newLine();
    _builder.append("\t\t    ");
    _builder.append("ts.tv_nsec = (periodic % 1000) * 1000000;");
    _builder.newLine();
    _builder.append("\t\t    ");
    _builder.append("nanosleep(&ts, NULL);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("#else");
    _builder.newLine();
    _builder.append("\t\t    ");
    _builder.append("usleep(periodic * 1000);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("#endif");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("if (oldValue != newValue) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("//the expression is changed, push an event to the queue");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append(SMCodeGeneratorConstants.EVENT_QUEUE, "\t\t");
    _builder.append(".push(statemachine::PRIORITY_2, NULL, id, statemachine::CHANGE_EVENT, 0);");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("oldValue = newValue;");
    _builder.newLine();
    _builder.append("}");
    this.core.createOpaqueBehavior(this.superContext, changeEventOp, _builder.toString());
  }

  public String getChangeEventExpression(final ChangeEvent e) {
    if (((e.getChangeExpression() == null) || (!(e.getChangeExpression() instanceof OpaqueExpression)))) {
      return "false";
    } else {
      ValueSpecification _changeExpression = e.getChangeExpression();
      return IterableExtensions.<String>head(((OpaqueExpression) _changeExpression).getBodies());
    }
  }
}
